from functools import partial
from importlib.util import find_spec

from fauxai.constants import API_KEY, ENDPOINT, NOT_AVAILABLE

__all__ = [
    "OpenAI",
    "AsyncOpenAI",
    "Client",
    "AsyncClient",
]

OpenAI = NOT_AVAILABLE
AsyncOpenAI = NOT_AVAILABLE

Client = NOT_AVAILABLE
AsyncClient = NOT_AVAILABLE

if find_spec("openai"):
    from openai import (
        AsyncClient,
        AsyncOpenAI,
        Client,
        OpenAI,
    )

    OpenAI = partial(OpenAI, base_url=ENDPOINT, api_key=API_KEY)
    AsyncOpenAI = partial(AsyncOpenAI, base_url=ENDPOINT, api_key=API_KEY)

    Client = partial(Client, base_url=ENDPOINT, api_key=API_KEY)
    AsyncClient = partial(AsyncClient, base_url=ENDPOINT, api_key=API_KEY)



import json
import os
from warnings import warn

from fastapi import FastAPI, Request
from starlette.exceptions import HTTPException
from starlette.responses import StreamingResponse

from fauxai.server.models import ChatCompletionsRequest
from fauxai.server.responses import (
    _normal_function_call,
    _normal_response,
    _streaming_function_call,
    _streaming_response,
)

app = FastAPI()


@app.post("/chat")
@app.post("/v1/messages")
@app.post("/chat/completions")
@app.post("/v1/chat/completions")
def chat_completions_create(request: Request, data: ChatCompletionsRequest):
    if data.messages:
        content = data.messages[-1].content
    elif data.message:
        content = data.message
    else:
        raise HTTPException(400, "No message content was found.")

    responses = os.getenv("FAUXAI_RESPONSES")

    FUNCTION_CALL = True if "func" in content.lower() else False
    STREAM = data.stream

    if request.url._url[-4:] == "chat":
        STREAM = False

    name = "mock_function"
    arguments = {"mock_arg": "mock_var"}

    if responses is not None:
        all_responses_dict = json.loads(responses)
        try:
            response_dict = all_responses_dict[content]
            try:
                response_type = response_dict["type"]
                if response_type == "function":
                    try:
                        name = response_dict["name"]
                        arguments = response_dict["arguments"]
                        FUNCTION_CALL = True
                    except KeyError:
                        raise HTTPException(
                            400, "Mock function call must have name and arguments field"
                        )
                elif response_type == "completion":
                    try:
                        content = response_dict["content"]
                        FUNCTION_CALL = False
                    except KeyError:
                        raise HTTPException(
                            400, "Mock completion must have content field"
                        )
            except KeyError:
                raise HTTPException(400, "Type of mock response must be specified")
        except KeyError:
            warn("No matching response found in JSON file, using default values...")

    model = data.model if data.model is not None else "mock-model"
    options = (FUNCTION_CALL, STREAM)

    match options:
        case (True, True):
            return StreamingResponse(_streaming_function_call(name, arguments, model))
        case (True, False):
            return _normal_function_call(name, arguments, model)
        case (False, True):
            return StreamingResponse(_streaming_response(content, model))
        case (False, False):
            return _normal_response(content, model)



import os
import subprocess
from typing import Annotated, Optional

from typer import Argument, FileText, Typer, echo

cli = Typer()

dir_path = os.path.dirname(os.path.realpath(__file__))


@cli.command()
def start(
    responses: Annotated[Optional[FileText], Argument()] = None,
    host: str = "127.0.0.1",
    port: int = 8100,
):
    if responses:
        echo(f"Reading {responses.name}...")
        os.environ["FAUXAI_RESPONSES"] = responses.read()

    echo(f"Starting FauxAI server on http://{host}:{port}...")
    subprocess.run(
        [
            "uvicorn",
            "--app-dir",
            f"{dir_path}",
            "server:app",
            "--host",
            host,
            "--port",
            str(port),
        ]
    )



from json import dumps
from time import time

__all__ = [
    "_normal_response",
    "_normal_function_call",
    "_streaming_response",
    "_streaming_function_call",
]


def _normal_response(content: str, model: str):
    response = {
        "id": "Null",
        "text": content,
        "content": content,
        "role": "assistant",
        "generation_id": "Null",
        "choices": [
            {
                "index": 0,
                "message": {"role": "assistant", "content": content},
                "finish_reason": "stop",
            }
        ],
        "created": int(time()),
        "model": model,
        "object": "chat.completion",
        "usage": {"prompt_tokens": 0, "completion_tokens": 0, "total_tokens": 0},
    }
    return response


def _normal_function_call(name: str, arguments: dict, model: str):
    response = {
        "id": "Null",
        "text": "None",
        "content": [
            {"type": "tool_use", "id": "None", "name": name, "input": arguments}
        ],
        "role": "assistant",
        "generation_id": "Null",
        "choices": [
            {
                "index": 0,
                "message": {
                    "role": "assistant",
                    "content": "None",
                    "tool_calls": [
                        {
                            "id": "Null",
                            "type": "function",
                            "function": {
                                "arguments": dumps(arguments),
                                "name": name,
                            },
                        }
                    ],
                },
                "finish_reason": "tool_calls",
            }
        ],
        "created": int(time()),
        "model": model,
        "object": "chat.completion",
        "usage": {"prompt_tokens": 0, "completion_tokens": 0, "total_tokens": 0},
        "tool_calls": [{"name": name, "parameters": arguments}],
    }
    return response


def _streaming_response(content: str, model: str):
    for char in content:
        chunk = {
            "id": "Null",
            "text": char,
            "content": None,
            "role": "assistant",
            "object": "chat.completion.chunk",
            "created": int(time()),
            "model": model,
            "choices": [
                {
                    "index": 0,
                    "delta": {"content": char, "role": "assistant"},
                    "finish_reason": "stop",
                }
            ],
            "type": "content_block_delta",
            "index": 0,
            "delta": {"type": "text_delta", "text": char},
        }
        yield f"data: {dumps(chunk)}\n\n"
    yield "data: [DONE]\n\n"


def _streaming_function_call(name: str, arguments: dict, model: str):
    for _ in name:
        chunk = {
            "id": "Null",
            "role": "assistant",
            "content": [
                {"type": "tool_use", "id": "None", "name": name, "input": arguments}
            ],
            "object": "chat.completion.chunk",
            "created": int(time()),
            "model": model,
            "choices": [
                {
                    "index": 0,
                    "delta": {
                        "role": "assistant",
                        "content": None,
                        "tool_calls": [
                            {
                                "id": "Null",
                                "type": "function",
                                "function": {
                                    "arguments": dumps(arguments),
                                    "name": name,
                                },
                            }
                        ],
                    },
                    "finish_reason": "tool_calls",
                }
            ],
            "tool_calls": [{"name": name, "parameters": arguments}],
        }
        yield f"data: {dumps(chunk)}\n\n"
    yield "data: [DONE]\n\n"



import sys
from types import ModuleType

from fauxai.clients import anthropic, cohere, mistralai, openai

clients = [
    (openai, openai.__all__),
    (mistralai, mistralai.__all__),
    (cohere, cohere.__all__),
    (anthropic, anthropic.__all__),
]

for sdk, client_classes in clients:
    sdk_name = sdk.__name__.replace("fauxai.clients.", "")
    mod = ModuleType(f"fauxai.{sdk_name}")

    sys.modules[mod.__name__] = mod

    for client in client_classes:
        sdk_client = getattr(sdk, client)
        setattr(mod, client, sdk_client)



from functools import partial
from importlib.util import find_spec

from fauxai.constants import API_KEY, ENDPOINT, NOT_AVAILABLE

__all__ = ["MistralClient", "MistralAsyncClient"]

MistralClient = NOT_AVAILABLE

if find_spec("mistralai"):
    from mistralai.async_client import MistralAsyncClient
    from mistralai.client import MistralClient

    MistralClient = partial(MistralClient, endpoint=ENDPOINT, api_key=API_KEY)
    MistralAsyncClient = partial(MistralAsyncClient, endpoint=ENDPOINT, api_key=API_KEY)



from typing import Optional

from pydantic import BaseModel, ConfigDict


class Message(BaseModel):
    role: str
    content: str


class ChatCompletionsRequest(BaseModel):
    model: str
    message: Optional[str] = None
    messages: Optional[list[Message]] = None
    chat_history: Optional[list[Message]] = None
    stream: Optional[bool] = False
    tools: Optional[dict] = None
    tool_choice: Optional[str] = None

    model_config = ConfigDict(extra="allow")



from functools import partial
from importlib.util import find_spec

from fauxai.constants import API_KEY, ENDPOINT, NOT_AVAILABLE

__all__ = ["Client", "AsyncClient"]

Client = NOT_AVAILABLE
AsyncClient = NOT_AVAILABLE

if find_spec("cohere"):
    from cohere import AsyncClient, Client

    Client = partial(Client, base_url=ENDPOINT, api_key=API_KEY)
    AsyncClient = partial(AsyncClient, base_url=ENDPOINT, api_key=API_KEY)



from functools import partial
from importlib.util import find_spec

from fauxai.constants import API_KEY, ENDPOINT, NOT_AVAILABLE

__all__ = [
    "Anthropic",
    "AsyncAnthropic",
    "Client",
    "AsyncClient",
]

Anthropic = NOT_AVAILABLE
AsyncAnthropic = NOT_AVAILABLE

Client = NOT_AVAILABLE
AsyncClient = NOT_AVAILABLE

if find_spec("anthropic"):
    from anthropic import Anthropic, AsyncAnthropic, AsyncClient, Client

    Anthropic = partial(Anthropic, base_url=ENDPOINT, api_key=API_KEY)
    AsyncAnthropic = partial(AsyncAnthropic, base_url=ENDPOINT, api_key=API_KEY)

    Client = partial(Client, base_url=ENDPOINT, api_key=API_KEY)
    AsyncClient = partial(AsyncClient, base_url=ENDPOINT, api_key=API_KEY)
